--[[
Name: TheSocial-1.0
Revision: $Rev: 44037 $
Author: David Lynch (kemayo@gmail.com)
Website: http://wiki.wowace.com/index.php/TheSocial-1.0
Documentation: http://wiki.wowace.com/index.php/TheSocial-1.0
SVN: svn://svn.wowace.com/wowace/trunk/TheSocial-1.0/TheSocial-1.0
Description: A library to provide an easy way to get information about your friend list.
Dependencies: AceLibrary, AceOO-2.0, AceEvent-2.0, Babble-Zone-2.2 (optional),
				Babble-Class-2.2 (optional)
License: LGPL v2.1
]]

local MAJOR_VERSION = "TheSocial-1.0"
local MINOR_VERSION = "$Revision: 44037 $"

-- Return right away if this isn't an upgrade.
if not AceLibrary then error(MAJOR_VERSION .. " requires AceLibrary.") end
if not AceLibrary:IsNewVersion(MAJOR_VERSION, MINOR_VERSION) then return end

if not AceLibrary:HasInstance("AceOO-2.0") then error(MAJOR_VERSION .. " requires AceOO-2.0.") end
if not AceLibrary:HasInstance("AceEvent-2.0") then error(MAJOR_VERSION .. " requires AceEvent-2.0.") end

local AceEvent = AceLibrary:GetInstance("AceEvent-2.0")
local BZ, BC

local TheSocial = {}

local ShowFriends = ShowFriends
local select = select
local next = next

local new, del
do
	local cache = {}
	function new(...)
		local t = next(cache)
		if t then
			cache[t] = nil
			for i = 1, select('#', ...) do
				t[i] = select(i, ...)
			end
			return t
		else
			return { ... }
		end
	end
	
	function del(t)
		for k in pairs(t) do
			t[k] = nil
		end
		cache[t] = true
		return nil
	end
end

local players = {}
local playersOnline = {}
local playerLevels = {}
local playerClasses = {}
local playerZones = {}
local playerStatuses = {}
local numPlayersOnline = 0
local numPlayersTotal = 0

local playersOnlineOld, playersOld, playerLevelsOld = {}, {}, {}
function TheSocial:FRIENDLIST_UPDATE()
	playersOnline, playersOnlineOld = playersOnlineOld, playersOnline -- for signon/off events
	players, playersOld = playersOld, players -- for add/remove events
	playerLevels, playerLevelsOld = playerLevelsOld, playerLevels -- for ding events
	numPlayersOnline = 0
	numPlayersTotal = GetNumFriends()
	
	for i = 1, numPlayersTotal do
		local name, level, class, zone, online, status = GetFriendInfo(i)
		local add, connect, ding
		if name then
			if status == "" then
				status = nil
			end
			players[name] = true
			playerLevels[name] = level or 0 -- note: 0 if offline
			playerClasses[name] = class or UNKNOWN
			playerZones[name] = zone or UNKNOWN
			playerStatuses[name] = status
			if playersOld[name] then
				playersOld[name] = nil
				if online and playerLevelsOld[name] and playerLevelsOld[name] > 0 and playerLevelsOld[name] < level then
					ding = true
				end
				playerLevelsOld[name] = nil
			else
				add = true
			end
		end
		if online then
			numPlayersOnline = numPlayersOnline + 1
			if name then
				playersOnline[name] = true
				if playersOnlineOld[name] then
					playersOnlineOld[name] = nil
				else
					connect = true
				end
			end
		end
		if add then
			self:TriggerEvent("TheSocial10_FriendAdded", name)
		end
		if connect then
			self:TriggerEvent("TheSocial10_FriendConnected", name)
		end
		if ding then
			self:TriggerEvent("TheSocial10_FriendLevelUp", name, level)
		end
	end
	for k in pairs(playersOld) do
		playersOld[k] = nil
		self:TriggerEvent("TheSocial10_FriendRemoved", k)
	end
	for k in pairs(playersOnlineOld) do
		playersOnlineOld[k] = nil
		self:TriggerEvent("TheSocial10_FriendDisconnected", k)
	end
	for k in pairs(playerLevelsOld) do
		playerLevelsOld[k] = nil
	end
	
	self:ScheduleEvent("TheSocial10-ShowFriends", ShowFriends, 15)
	self:TriggerEvent("TheSocial10_Updated")
end

function TheSocial:GetNumOnline()
	return numPlayersOnline
end

function TheSocial:GetNumTotal()
	return numPlayersTotal
end

function TheSocial:HasFriend(name)
	return players[name] and true or false
end

function TheSocial:IsFriendOnline(name)
	return playersOnline[name] or false
end

function TheSocial:GetLevel(name)
	return playerLevels[name]
end

function TheSocial:GetClass(name)
	return playerClasses[name]
end

function TheSocial:GetEnglishClass(name)
	if not BC then
		self:error("Cannot call `GetEnglishClass' without Babble-Class-2.2 loaded.")
	end
	local class = playerClasses[name]
	if class then
		BC:GetReverseTranslation(name)
	else
		return nil
	end
end

function TheSocial:GetClassColor(name)
	if not BC then
		self:error("Cannot call `GetClassColor' without Babble-Class-2.2 loaded.")
	end
	local class = playerClasses[name]
	if class then
		return BC:GetColor(class)
	else
		return 0.8, 0.8, 0.8
	end
end

function TheSocial:GetClassHexColor(name)
	if not BC then
		self:error("Cannot call `GetClassHexColor' without Babble-Class-2.2 loaded.")
	end
	local class = playerClasses[name]
	if class then
		return BC:GetHexColor(class)
	else
		return "cccccc"
	end
end

function TheSocial:GetZone(name)
	return playerZones[name]
end

function TheSocial:GetEnglishZone(name)
	if not BZ then
		self:error("Cannot call `GetEnglishZone' without Babble-Zone-2.2 loaded.")
	end
	local zone = playerZones[name]
	if zone ~= UNKNOWN then
		return BZ:GetReverseTranslation(zone)
	else
		return "Unknown"
	end
end

function TheSocial:GetStatus(name)
	return playerStatuses[name]
end

local sorts; sorts = {
	NAME = function(a,b)
		return a < b
	end,
	LEVEL = function(a,b)
		if playerLevels[a] == playerLevels[b] then
			return sorts.NAME(a,b)
		else
			return playerLevels[a] < playerLevels[b]
		end
	end,
	CLASS = function(a,b)
		if playerClasses[a] == playerClasses[b] then
			return sorts.LEVEL(a,b)
		else
			return playerClasses[a] < playerClasses[b]
		end
	end,
	ZONE = function(a,b)
		if playerZones[a] == playerZones[b] then
			return sorts.CLASS(a,b)
		else
			return playerZones[a] < playerZones[b]
		end
	end,
	ONLINE = function(a,b)
		if playersOnline[a] == playersOnline[b] then
			return sorts.CLASS(a,b)
		else
			return connected[a] --1 or nil, you see.
		end
	end,
}

local iter = function(t)
	local n = t.n
	n = n + 1
	t.n = n
	return t[n] or del(t)
end
function TheSocial:GetIterator(sort, includeOffline)
	self:argCheck(sort, 2, "string", "nil")
	local sortFunc = sorts[sort or "NAME"]
	if not sortFunc then
		self:error('Argument #2 must be "NAME", "LEVEL", "CLASS", "ZONE", "ONLINE", or nil, got %q.', sort)
	end
	
	local tmp = new()
	for k in pairs(tmp) do
		tmp[k] = nil
	end
	
	for k in pairs(includeOffline and players or playersOnline) do
		tmp[#tmp+1] = k
	end
	
	table.sort(tmp, sortFunc)
	tmp.n = 0
	
	return iter, tmp, nil
end

local function external(self, major, instance)
	if major == "Babble-Class-2.2" then
		BC = instance
	elseif major == "Babble-Zone-2.2" then
		BZ = instance
	end
end

local function activate(self, oldLib, oldDeactivate)
	TheSocial = self
	
	AceEvent:embed(self)
	
	self:UnregisterAllEvents()
	
	self:ScheduleEvent("TheSocial10-ShowFriends", ShowFriends, 15)
	self:RegisterBucketEvent("FRIENDLIST_UPDATE", 1)
	
	if oldDeactivate then
		oldDeactivate(oldLib)
	end
end

AceLibrary:Register(TheSocial, MAJOR_VERSION, MINOR_VERSION, activate, nil, external)
